'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _defineProperty2 = require('babel-runtime/helpers/defineProperty');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _entries = require('babel-runtime/core-js/object/entries');

var _entries2 = _interopRequireDefault(_entries);

exports.default = function (type) {
  var fields = type._typeConfig.fields();
  return (0, _entries2.default)(fields).reduce(function (args, _ref) {
    var _ref2 = (0, _slicedToArray3.default)(_ref, 2);

    var key = _ref2[0];
    var field = _ref2[1];

    return (0, _assign2.default)(args, fieldToArg(key, field));
  }, {});
};

var _graphql = require('graphql');

var _customType = require('../type/customType');

var _pluralize = require('pluralize');

var _pluralize2 = _interopRequireDefault(_pluralize);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var fieldToArg = function fieldToArg(key, field) {
  var typeName = field.type.name || field.type.constructor.name;
  var required = field.required;
  var graphqlType = void 0;
  if (typeName !== 'GraphQLList') {
    graphqlType = nameToType(typeName, field);
    // Custom type for Object attribute in mongoose model. e.g. {name: {first, last}}
    if (!graphqlType) return buildObjectArgs(key, field);
    return buildArgs(key, graphqlType, required);
  } else {
    // Deal with List type
    graphqlType = nameToType(field.type.ofType.name, field);
    return (0, _defineProperty3.default)({}, key, { type: new _graphql.GraphQLList(graphqlType), onlyPlural: true, required: required });
  }
};

/**
 * Generate args based on type's fields
 * @parmas
 *  - tyoe {Object} built graphql type
 * @return
 *  - defaultArgs {Object} argus based on type's fields, including singular and plural
 * @notice
 *  - response has `id` and `ids` instead of _id
 *    mongoose query should convert id to _id or mongoose won't support it
 *  - name.first in Mongoose model will have args 'name_first' and 'name_firsts'
 *    because graphql name convention only support _a-zA-Z0-9
 */


var buildArgs = function buildArgs(key, graphqlType, required) {
  var _ref5;

  if ((0, _keys2.default)(graphqlType).length === 0) return {};
  var plural = _pluralize2.default.plural(key);
  var isPlural = plural === key;
  return isPlural ? (0, _defineProperty3.default)({}, key, { type: new _graphql.GraphQLList(graphqlType), onlyPlural: true, required: required }) : (_ref5 = {}, (0, _defineProperty3.default)(_ref5, key, { type: graphqlType, required: required }), (0, _defineProperty3.default)(_ref5, plural, { type: new _graphql.GraphQLList(graphqlType) }), _ref5);
};

var nameToType = function nameToType(typeName, field) {
  var hasResolve = !!field.resolve;
  switch (typeName) {
    case 'ID':
      return _graphql.GraphQLID;
    case 'String':
      return _graphql.GraphQLString;
    case 'Float':
      return _graphql.GraphQLFloat;
    case 'Boolean':
      return _graphql.GraphQLBoolean;
    case 'Buffer':
      return _customType.GraphQLBuffer;
    case 'Date':
      return _customType.GraphQLDate;
    case 'Mixed':
      return {};
    default:
      if (hasResolve) return _graphql.GraphQLID; // other models, use ID as reference
      return null; // Object attribute in mongoose model
  }
};

// Build args for Object attribute of the mongoose model
var buildObjectArgs = function buildObjectArgs(parentKey, field) {
  var fields = field.type._typeConfig.fields();
  return (0, _entries2.default)(fields).map(function (_ref6) {
    var _ref7 = (0, _slicedToArray3.default)(_ref6, 2);

    var key = _ref7[0];
    var value = _ref7[1];

    return fieldToArg(parentKey + '_' + key, value);
  }).reduce(function (args, myArg) {
    return (0, _assign2.default)(args, myArg);
  }, {});
};