'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _entries = require('babel-runtime/core-js/object/entries');

var _entries2 = _interopRequireDefault(_entries);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _defineProperty2 = require('babel-runtime/helpers/defineProperty');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

exports.default = function (model, type) {
  var _ref;

  var modelName = model.modelName;
  var defaultArgs = (0, _buildArgs2.default)(type);
  return _ref = {}, (0, _defineProperty3.default)(_ref, 'create' + modelName, buildCreate(model, type, defaultArgs)), (0, _defineProperty3.default)(_ref, 'update' + modelName, buildUpdate(model, type, defaultArgs)), (0, _defineProperty3.default)(_ref, 'delete' + modelName, buildDelete(model, type, defaultArgs)), _ref;
};

var _graphql = require('graphql');

var _buildArgs = require('./buildArgs');

var _buildArgs2 = _interopRequireDefault(_buildArgs);

var _utils = require('../utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var buildCreate = function buildCreate(Model, type, defaultArgs) {
  return {
    type: type,
    args: (0, _utils.filterArgs)(defaultArgs, { id: true, plural: true }),
    resolve: function () {
      var _ref2 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee(_, args) {
        var instance;
        return _regenerator2.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                instance = new Model((0, _utils.toMongooseArgs)(args));
                _context.next = 3;
                return instance.save();

              case 3:
                return _context.abrupt('return', _context.sent);

              case 4:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, undefined);
      }));

      return function resolve(_x, _x2) {
        return _ref2.apply(this, arguments);
      };
    }()
  };
};

/**
 * Build mutation for single model
 * @params
 *  - model a mongoose model
 *  - type a corresponding converted graphql type
 * @return
 *  - {Object} e.g. { createUser: {type: userType, args, resolve}, updateUser, removeUser }
 */


var buildUpdate = function buildUpdate(Model, type, defaultArgs) {
  return {
    type: type,
    args: (0, _utils.filterArgs)(defaultArgs, { plural: true, required: true, idRequired: true }),
    resolve: function () {
      var _ref3 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2(_, args) {
        var updateData;
        return _regenerator2.default.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                updateData = (0, _entries2.default)(args).filter(function (_ref4) {
                  var _ref5 = (0, _slicedToArray3.default)(_ref4, 2);

                  var key = _ref5[0];
                  var _ = _ref5[1];

                  if (key === 'id' || key === 'ids') return false;
                  return true;
                }).map(function (_ref6) {
                  var _ref7 = (0, _slicedToArray3.default)(_ref6, 2);

                  var key = _ref7[0];
                  var value = _ref7[1];

                  return [key.replace('_', '.'), value];
                }).reduce(function (args, _ref8) {
                  var _ref9 = (0, _slicedToArray3.default)(_ref8, 2);

                  var key = _ref9[0];
                  var value = _ref9[1];
                  return (0, _assign2.default)(args, (0, _defineProperty3.default)({}, key, value));
                }, {});
                _context2.next = 3;
                return Model.update({ _id: args.id }, { $set: updateData });

              case 3:
                _context2.next = 5;
                return Model.findById(args.id);

              case 5:
                return _context2.abrupt('return', _context2.sent);

              case 6:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, undefined);
      }));

      return function resolve(_x3, _x4) {
        return _ref3.apply(this, arguments);
      };
    }()
  };
};

var buildDelete = function buildDelete(Model, type, defaultArgs) {
  var returnType = new _graphql.GraphQLObjectType({
    name: Model.modelName + 'deleteMutationReturn',
    fields: function fields() {
      return {
        success: { type: _graphql.GraphQLBoolean },
        msg: { type: _graphql.GraphQLString }
      };
    }
  });
  return {
    type: returnType,
    args: (0, _utils.filterArgs)(defaultArgs, { plural: true, idRequired: true, onlyId: true }),
    resolve: function () {
      var _ref10 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee3(_, args) {
        var res;
        return _regenerator2.default.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                res = { success: true, msg: null };
                _context3.prev = 1;
                _context3.next = 4;
                return Model.findById(args.id).remove();

              case 4:
                _context3.next = 9;
                break;

              case 6:
                _context3.prev = 6;
                _context3.t0 = _context3['catch'](1);

                res = { success: false, msg: _context3.t0.message };

              case 9:
                return _context3.abrupt('return', res);

              case 10:
              case 'end':
                return _context3.stop();
            }
          }
        }, _callee3, undefined, [[1, 6]]);
      }));

      return function resolve(_x5, _x6) {
        return _ref10.apply(this, arguments);
      };
    }()
  };
};