'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

exports.buildSchema = buildSchema;

var _buildQuery = require('./buildQuery');

var _buildQuery2 = _interopRequireDefault(_buildQuery);

var _buildMutation = require('./buildMutation');

var _buildMutation2 = _interopRequireDefault(_buildMutation);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Build graphql CRUD schema based on models and types
 * @params
 *  - models {Array} mongoose models
 *  - typeMap {Object} map of model and corresponding graphql type
 * @return
 *  - grapqhl schema which has query and mutation
 */
function buildSchema(models, typeMap) {
  var type = void 0;

  var _models$map$reduce = models.map(function (model) {
    type = typeMap[model.modelName];
    return {
      query: (0, _buildQuery2.default)(model, type),
      mutation: (0, _buildMutation2.default)(model, type)
    };
  }).reduce(function (fields, modelField) {
    fields.query = (0, _assign2.default)({}, fields.query, modelField.query);
    fields.mutation = (0, _assign2.default)({}, fields.mutation, modelField.mutation);
    return fields;
  }, { query: {}, mutation: {} });

  var query = _models$map$reduce.query;
  var mutation = _models$map$reduce.mutation;


  return {
    query: query,
    mutation: mutation
  };
}