'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.GraphQLMixed = exports.GraphQLDate = exports.GraphQLBuffer = undefined;

var _graphql = require('graphql');

/**
 * Buffer type
 */
var coerceBuffer = function coerceBuffer(value) {
  if (value instanceof Buffer) return value;
  throw new TypeError('Type error: ' + value + ' is not instance of Buffer');
};

var GraphQLBuffer = exports.GraphQLBuffer = new _graphql.GraphQLScalarType({
  name: 'Buffer',
  serialize: coerceBuffer, // serialize to query result
  parseValue: coerceBuffer,
  parseLiteral: function parseLiteral(ast) {
    // Read from args
    return typeof ast.value === 'string' && new Buffer(ast.value) || null;
  }
});

/**
 * Date type
 */
var coerceDate = function coerceDate(value) {
  if (value instanceof Date) return value;
  throw new TypeError('Type error: ' + value + ' is not instance of Date');
};

var GraphQLDate = exports.GraphQLDate = new _graphql.GraphQLScalarType({
  name: 'Date',
  serialize: coerceDate,
  parseValue: coerceDate,
  parseLiteral: function parseLiteral(ast) {
    var d = new Date(ast.value);
    return !isNaN(d.getTime()) && d || null;
  }
});

/**
 * Mixed type
 */
var GraphQLMixed = exports.GraphQLMixed = new _graphql.GraphQLScalarType({
  name: 'Mixed',
  serialize: function serialize(value) {
    return value;
  },
  parseValue: function parseValue(value) {
    return value;
  },
  parseLiteral: function parseLiteral(ast) {
    return ast.value;
  }
});