'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _entries = require('babel-runtime/core-js/object/entries');

var _entries2 = _interopRequireDefault(_entries);

var _defineProperty2 = require('babel-runtime/helpers/defineProperty');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

exports.modelsToTypes = modelsToTypes;

var _graphql = require('graphql');

var _customType = require('./customType');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _typeMap = {};
/**
 * Convert bundch of mongoose model to graphql types
 * build this as singleton so that it won't create graphQLType twice
 *
 * @params
 *  - models {Array} list of mongoose models
 * @return
 *  - typeMap {Object} key: modelName, value: type
 */
function modelsToTypes(models) {
  var _this = this;

  var typeMap = models.filter(function (model) {
    if (_typeMap[model.modelName]) return false;
    return true;
  }).reduce(function (map, model) {
    return (0, _assign2.default)(map, (0, _defineProperty3.default)({}, model.modelName, toType(model)));
  }, {});
  _typeMap = (0, _assign2.default)(_typeMap, typeMap);

  // Deal with ref after all types are defined
  (0, _entries2.default)(typeMap).forEach(function (_ref) {
    var _ref2 = (0, _slicedToArray3.default)(_ref, 2);

    var modelName = _ref2[0];
    var type = _ref2[1];

    var originFileds = type._typeConfig.fields();
    var newTypeFileds = (0, _entries2.default)(originFileds).map(function (_ref3) {
      var _ref4 = (0, _slicedToArray3.default)(_ref3, 2);

      var path = _ref4[0];
      var pathValue = _ref4[1];

      if (pathValue.ref) {
        var _ret = function () {
          var ref = pathValue.ref;
          if (!_typeMap[ref]) throw TypeError(ref + ' is not a model');
          var model = models.find(function (m) {
            return m.modelName === ref;
          });
          var refModelType = _typeMap[ref];
          if (pathValue.type instanceof _graphql.GraphQLList) {
            return {
              v: (0, _defineProperty3.default)({}, path, {
                type: new _graphql.GraphQLList(refModelType),
                resolve: function () {
                  var _ref5 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee(instance) {
                    return _regenerator2.default.wrap(function _callee$(_context) {
                      while (1) {
                        switch (_context.prev = _context.next) {
                          case 0:
                            _context.next = 2;
                            return model.find({ _id: { $in: instance[path] } });

                          case 2:
                            return _context.abrupt('return', _context.sent);

                          case 3:
                          case 'end':
                            return _context.stop();
                        }
                      }
                    }, _callee, _this);
                  }));

                  return function resolve(_x) {
                    return _ref5.apply(this, arguments);
                  };
                }()
              })
            };
          } else {
            return {
              v: (0, _defineProperty3.default)({}, path, {
                type: refModelType,
                resolve: function () {
                  var _ref6 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2(instance) {
                    return _regenerator2.default.wrap(function _callee2$(_context2) {
                      while (1) {
                        switch (_context2.prev = _context2.next) {
                          case 0:
                            _context2.next = 2;
                            return model.findById(instance[path]);

                          case 2:
                            return _context2.abrupt('return', _context2.sent);

                          case 3:
                          case 'end':
                            return _context2.stop();
                        }
                      }
                    }, _callee2, _this);
                  }));

                  return function resolve(_x2) {
                    return _ref6.apply(this, arguments);
                  };
                }()
              })
            };
          }
        }();

        if ((typeof _ret === 'undefined' ? 'undefined' : (0, _typeof3.default)(_ret)) === "object") return _ret.v;
      }
      return (0, _defineProperty3.default)({}, path, pathValue);
    }).reduce(function (typeField, path) {
      return (0, _assign2.default)(typeField, path);
    }, {});

    typeMap[modelName]._typeConfig.fields = function () {
      return newTypeFileds;
    };
  });

  _typeMap = (0, _assign2.default)(_typeMap, typeMap);
  return _typeMap;
}

/* Convert a mongoose model to corresponding type */
var toType = function toType(model) {
  var exceptPath = ['_id', '__v'];
  var paths = model.schema.paths;
  var _fields = (0, _keys2.default)(paths).filter(function (path) {
    return exceptPath.indexOf(path) === -1;
  }).map(function (path) {
    var attr = paths[path];
    var field = { type: pathToType(attr) };
    // Find out ref on mongoose model's path, use subPath's ref if path is an Array
    if (attr.options.ref || attr.instance === 'Array' && attr.caster.options.ref) {
      field.ref = attr.options.ref || attr.caster.options.ref;
    }
    // Mark required path
    var required = attr.options.required;
    if (Array.isArray(required) && required[0] || required) field.required = true;
    return (0, _defineProperty3.default)({}, path, field);
  }).reduce(function (fields, path) {
    // make up object tpe, e.g { name: { first: {type: GraphQLString...}, last: {type: GraphQLString...} } }
    var pathKey = (0, _keys2.default)(path)[0];
    var pathKeySplit = pathKey.split('.');
    var pathKeyLength = pathKeySplit.length;
    if (pathKeyLength.length === 1) return (0, _assign2.default)(fields, path);
    pathKeySplit.reduce(function (fieldPostion, depth, index) {
      if (index === pathKeyLength - 1) {
        fieldPostion[depth] = path[pathKey];
        return;
      }
      fieldPostion[depth] = fieldPostion[depth] || {};
      return fieldPostion[depth];
    }, fields);
    return fields;
  }, { id: { type: _graphql.GraphQLID } });

  // Deal with object attribute in mongoose model
  // e.g. {name: {first: String, last: Strinf}} -> {name: GraphQLType{fields: {first: GraphQLString, two: GraphQLString}}}
  _fields = (0, _entries2.default)(_fields).map(function (_ref9) {
    var _ref10 = (0, _slicedToArray3.default)(_ref9, 2);

    var key = _ref10[0];
    var attr = _ref10[1];

    return (0, _defineProperty3.default)({}, key, convertObject(attr, '' + model.modelName + key));
  }).reduce(function (fields, path) {
    return (0, _assign2.default)(fields, path);
  }, {});
  return new _graphql.GraphQLObjectType({
    name: model.modelName,
    fields: function fields() {
      return _fields;
    }
  });
};

// Convert single path of mongoose to type
var pathToType = function pathToType(path) {
  switch (path.instance) {
    case 'String':
      return _graphql.GraphQLString;
    case 'Number':
      // Float includes Int
      // @see https://github.com/graphql/graphql-js/blob/master/src/type/scalars.js#L69
      return _graphql.GraphQLFloat;
    case 'Date':
      return _customType.GraphQLDate;
    case 'Buffer':
      return _customType.GraphQLBuffer;
    case 'Boolean':
      return _graphql.GraphQLBoolean;
    case 'Mixed':
      return _customType.GraphQLMixed;
    case 'ObjectID':
      return _graphql.GraphQLID;
    case 'Array':
      var subType = pathToType(path.caster);
      return new _graphql.GraphQLList(subType);
    default:
      return _customType.GraphQLMixed;
  }
};

// Convert model's object attribute
var convertObject = function convertObject(attr, parentName) {
  if (attr.type && (attr.type instanceof _graphql.GraphQLScalarType || attr.type instanceof _graphql.GraphQLList)) return attr;
  var _fields2 = (0, _entries2.default)(attr).map(function (_ref12) {
    var _ref13 = (0, _slicedToArray3.default)(_ref12, 2);

    var key = _ref13[0];
    var vakue = _ref13[1];

    return (0, _defineProperty3.default)({}, key, convertObject(attr[key], '' + parentName + key));
  }).reduce(function (fields, path) {
    // console.log(path)
    return (0, _assign2.default)(fields, path);
  }, {});
  // console.log(fields)
  return {
    type: new _graphql.GraphQLObjectType({
      name: '' + parentName,
      fields: function fields() {
        return _fields2;
      }
    })
  };
};