'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _defineProperty2 = require('babel-runtime/helpers/defineProperty');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _entries = require('babel-runtime/core-js/object/entries');

var _entries2 = _interopRequireDefault(_entries);

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

exports.filterArgs = filterArgs;
exports.toMongooseArgs = toMongooseArgs;

var _graphql = require('graphql');

var _pluralize = require('pluralize');

var _pluralize2 = _interopRequireDefault(_pluralize);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Filter arguments when doing CRUD
 * @params
 *  - defaultArgs {Object} the result of buildArgs
 *  - opt {Object {filter: {Bool}} options: id, plural, required, idRequired, onlyId
 */
function filterArgs(defaultArgs, opt) {
  opt = opt || {};
  var packValueToNonNull = function packValueToNonNull(value) {
    return (0, _assign2.default)({}, value, { type: new _graphql.GraphQLNonNull(value.type) });
  };
  return (0, _entries2.default)(defaultArgs).filter(function (_ref) {
    var _ref2 = (0, _slicedToArray3.default)(_ref, 2);

    var arg = _ref2[0];
    var value = _ref2[1];

    if (opt.onlyId && arg !== 'id' && arg !== 'ids') return false;
    if (opt.id && (arg === 'id' || arg === 'ids')) return false;
    if (opt.plural && !value.onlyPlural && _pluralize2.default.plural(arg) === arg) return false;
    return true;
  }).map(function (_ref3) {
    var _ref4 = (0, _slicedToArray3.default)(_ref3, 2);

    var arg = _ref4[0];
    var value = _ref4[1];

    var newValue = (0, _assign2.default)({}, value);
    if ((arg === 'id' || arg === 'ids') && opt.idRequired) newValue = packValueToNonNull(newValue);
    if (!opt.required && newValue.required) newValue = packValueToNonNull(newValue);
    return [arg, newValue];
  }).reduce(function (args, _ref5) {
    var _ref6 = (0, _slicedToArray3.default)(_ref5, 2);

    var arg = _ref6[0];
    var value = _ref6[1];

    return (0, _assign2.default)(args, (0, _defineProperty3.default)({}, arg, value));
  }, {});
}

/**
 * Convert args that graphql know to the args that mongoose know
 * so that the args can be used by mongoose to find or create
 */
function toMongooseArgs(args) {
  // Covert name_first to name: {first}
  var keyDepth = [];
  return (0, _entries2.default)(args).reduce(function (args, _ref7) {
    var _ref8 = (0, _slicedToArray3.default)(_ref7, 2);

    var key = _ref8[0];
    var value = _ref8[1];

    keyDepth = key.split('_');
    if (keyDepth.length === 1) return (0, _assign2.default)(args, (0, _defineProperty3.default)({}, key, value));
    keyDepth.reduce(function (args, depth, index) {
      if (index === keyDepth.length - 1) {
        args[depth] = value;
        return;
      }
      args[depth] = args[depth] || {};
      return args[depth];
    }, args);
    return args;
  }, {});
}